/*
 *  Linux Frame Buffer Device Configuration
 *
 *   Copyright 1995-1999 by Geert Uytterhoeven <geert@linux-m68k.org>
 *
 *  --------------------------------------------------------------------------
 *
 *  This file is subject to the terms and conditions of the GNU General Public
 *  License. See the file COPYING in the main directory of the Linux
 *  distribution for more details.
 */

%{

#define YYSTYPE		long

#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>

#include "config.tab.h"
#include "util.h"

extern int yyparse(void);

struct keyword {
    const char *name;
    int token;
    int value;
};

static struct keyword keywords[] = {
    { "geometry", GEOMETRY, 0 },
    { "format", FORMAT, 0 },
    { "mode", MODE, 0 },
    { "hsync", HSYNC, 0 },
    { "vsync", VSYNC, 0 },
    { "csync", CSYNC, 0 },
    { "gsync", GSYNC, 0 },
    { "extsync", EXTSYNC, 0 },
    { "broadcast", BCAST, 0 },
    { "interlace", LACED, 0 },
    { "doublescan", DOUBLE, 0 },
    { "low", POLARITY, 0 },
    { "high", POLARITY, 1 },
    { "false", BOOL, 0 },
    { "true", BOOL, 1 },
    { "", -1, 0 }
};

static int line = 1;

static const char *ConfigFile = NULL;


void ReadDatabase(const char *filename)
{
    ConfigFile = filename;

    if (!(yyin = fopen(filename, "r")))
	Die("fopen %s: %s\n", filename, strerror(errno));
    yyparse();
    fclose(yyin);
}

void __attribute__((noreturn)) yyerror(const char *fmt, ...)
{
    va_list ap;

    va_start(ap, fmt);
    fflush(stdout);
    fprintf(stderr, "%s:%d: ", ConfigFile, line);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
    Die("\n");
}

extern "C" int yywrap(void);
int yywrap(void)
{
    return 1;
}

static const char *CopyString(const char *s)
{
    int len = strlen(s);
    if (s[0] == '"') {
	s++;
	len -= 2;
    }
    char *s2 = new char[len+1];
    strncpy(s2, s, len);
    s2[len] = '\0';
    return s2;
}

static int FindToken(const char *s)
{
    int i;

    for (i = 0; keywords[i].token > 0; i++)
	if (!strcmp(s, keywords[i].name)) {
	    yylval = keywords[i].value;
	    return keywords[i].token;
	}
    yylval = (unsigned long)CopyString(s);
    return STRING;
}

%}

keyword	[a-zA-Z][a-zA-Z0-9]*
int	[0-9]*
string	\"[^\"\n]*\"
comment	\#([^\n]*)
space	[ \t]+
junk	.

%%

{keyword}	{
		    return FindToken(yytext);
		}

{int}		{
		    yylval = strtoul(yytext, NULL, 0);
		    return INT;
		}

{string}	{
		    yylval = (unsigned long)CopyString(yytext);
		    return STRING;
		}

{comment}$	break;

{space}		break;

\n		{
		    line++;
		    break;
		}

\{		{
		    return OPENB;
		}

\}		{
		    return CLOSEB;
		}

{junk}		{
		    yyerror("Invalid token `%s'\n", yytext);
		}

%%
